!> @file check_namelist_files.f90
!--------------------------------------------------------------------------------!
! This file is part of PALM.
!
! PALM is free software: you can redistribute it and/or modify it under the terms
! of the GNU General Public License as published by the Free Software Foundation,
! either version 3 of the License, or (at your option) any later version.
!
! PALM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! PALM. If not, see <http://www.gnu.org/licenses/>.
!
! Copyright 1997-2014 Leibniz Universitaet Hannover
!--------------------------------------------------------------------------------!
!
! Current revisions:
! -----------------
! 
! 
! Former revisions:
! -----------------
! $Id: check_namelist_files.f90 1683 2015-10-07 23:57:51Z knoop $
!
! 1682 2015-10-07 23:56:08Z knoop
! Code annotations made doxygen readable
!
! 1320 2014-03-20 08:40:49Z raasch
! ONLY-attribute added to USE-statements
!
! 1088 2013-01-21 09:26:07Z raasch
! Bugfix: output format changed
!
! 1036 2012-10-22 13:43:42Z raasch
! code put under GPL (PALM 3.9)
!
! 931 2012-06-08 15:09:28Z maronga
! Check of max_pr_user implemented
!
! 845 2012-03-07 10:23:05Z maronga
! Added call to init_cloud_physics
!
! 807 2012-01-25 11:53:51Z maronga
! Initial revision
!
! Description:
! ------------
!> Perform namelist file check. The program is independent from PALM and used by 
!> the shell script "mrun" to check the parameter files (e.g. p3d, p3df...) b
!> efore the job is submitted/started. Source code from the PALM model in used in 
!> check_namelist_files by means of the cpp directive "__check". The compiled
!> program resides in the SCRIPTS folder as check_namelist_files.x.
!------------------------------------------------------------------------------!
PROGRAM check_namelist_files
 


    USE control_parameters,                                                    &
        ONLY:  check_restart, max_pr_user

    USE pegrid


    IMPLICIT NONE


!
!-- Read number of processors, the variable check_restart and max_user_pr. 
!-- check_restart gives information whether the initalor the restart parameter
!-- file must be checked. max_user_pr is usually set by read_var_list
!-- (excluded in the namelist file check).
    READ (*,*,ERR=10,END=10)  numprocs, check_restart, max_pr_user
10  CONTINUE


!
!-- Read control parameters from NAMELIST files and read environment-variables1
    CALL parin

    OPEN( 11, FILE='parin_for_check', FORM='FORMATTED' )
    WRITE( 11, FMT='(I4)' )  max_pr_user
    CLOSE( 11 )
!
!-- Determine processor topology and local array indicesy

    CALL init_pegrid

!
!-- Generate grid parameters
    CALL init_grid

!
!-- Check control parameters and deduce further quantities    
    CALL check_parameters

!
!-- Check absolute temperature and pressure. Delete this call in case that
!-- init_3d_model is compiled for the parameter file check
    CALL init_cloud_physics

 END PROGRAM check_namelist_files
